/* This file contains the main program of the File System.  It consists of
 * a loop that gets messages requesting work, carries out the work, and sends
 * replies.
 *
 * The entry points into this file are
 *   main:      main program of the File System
 */
#include "test.cnf"
#include "lib\include\string.h"

#define EXTERN extern
#include "kernel/makra.h"
#include "kernel/proc_pub.h"
#include "common/callnr.h"
#include "common/type.h"
#include "common/com.h"
#include "common/const.h"
#include "common/stat.h"
#include "systask/syst_pub.h"
#undef EXTERN

#define EXTERN
#include "fs/common/fsconst.h"
#include "fs/common/file.h"
#include "fs/common/fproc.h"
#include "fs/common/fsglo.h"
#include "fs/inode/inode.h"
#include "fs/inode/super.h"
#include "fs/cache/cache.h"
//#include "fs/common/fproc.h"
#include "fs/table.c"
#include "ipx/ipx.h"
#undef EXTERN

#if defined(DEBUG_FS) || defined(REPORT_FS_INIT)
 #include "test\kprintf.h"
 Byte debugLine=INITREPORT_ROW+1;
 Byte initReportCol=INITREPORT_COL;
#endif

/* globalni promenne: */

extern int ipx_ready;

struct fproc_struct *fp;
int superuser;
int dont_reply; /* neposilat odpoved */
message fs_msg; /* - prijata zprava
		    podle ni dale nastaveny:*/

int who; /* - kdo zpravu odeslal (pozice v tab procesu) */
int fs_call; /* cislo volani */

message fs_repmsg; /* - zprava pro odpoved */
char user_path[PATH_MAX]; /*- zde se bude ukladat
		     jmeno souboru/adresare, prenesene z adr.
		     prostoru uzivatele do adr. prostoru FS
		     funkci fetch_name() */

void load_super(void);

/*===========================================================================*
 *                              reply                                        *
 *===========================================================================*/
PUBLIC void reply(int whom)
/* process to reply to */
{
/* Send a reply to a user process. It may fail (if the process has just
 * been killed by a signal), so don't check the return code.  If the send
 * fails, just ignore it.
 */

  send(whom, fs_repmsg);
}


/*===========================================================================*
 *                              fs_init                                      *
 *===========================================================================*/
PRIVATE void fs_init(void)
{
  int i;
/* Initialize global variables, tables, etc. */

  #ifdef REPORT_FS_INIT
   kprintf(INITREPORT_ROW,initReportCol,7,"%c",'0'+(initReportCol++-INITREPORT_COL));
  #endif
  buf_pool(); /* - inicializace buffer pool, implementovano v cache */

  #ifdef REPORT_FS_INIT
   kprintf(INITREPORT_ROW,initReportCol,7,"%c",'0'+(initReportCol++-INITREPORT_COL));
  #endif
  init_inodes();

  #ifdef REPORT_FS_INIT
   kprintf(INITREPORT_ROW,initReportCol,7,"%c",'0'+(initReportCol++-INITREPORT_COL));
  #endif
  load_super();

  #ifdef REPORT_FS_INIT
   kprintf(INITREPORT_ROW,initReportCol,7,"%c",'0'+(initReportCol++-INITREPORT_COL));
  #endif
  ipx_init();

  #ifdef REPORT_FS_INIT
   kprintf(INITREPORT_ROW,initReportCol,7,"%c",'0'+(initReportCol++-INITREPORT_COL));
  #endif

/*inicializace slotu fproc[MM_POS,FS_POS,INIT_POS]:*/
  fproc[MM].fp_umask=~0;
  fproc[MM].fp_workdir=super_blocks[0].s_isup;
  fproc[MM].fp_rootdir=super_blocks[0].s_isup;
  for (i=0; i<NR_FDS; i++)
    fproc[MM].fp_filp[i]=NIL_FILP;
  fproc[MM].fp_suspended=NOT_SUSPENDED;
  fproc[MM].fp_revived=NOT_REVIVING;
  fproc[MM].fp_realuid=SYS_UID;
  fproc[MM].fp_effuid=SYS_UID;
  fproc[MM].fp_realgid=SYS_GID;
  fproc[MM].fp_realgid=SYS_GID;

  fproc[FS].fp_umask=~0;
  fproc[FS].fp_workdir=super_blocks[0].s_isup;
  fproc[FS].fp_rootdir=super_blocks[0].s_isup;

  for (i=0; i<NR_FDS; i++)
    fproc[FS].fp_filp[i]=NIL_FILP;
  fproc[FS].fp_suspended=NOT_SUSPENDED;
  fproc[FS].fp_revived=NOT_REVIVING;
  fproc[FS].fp_realuid=SYS_UID;
  fproc[FS].fp_effuid=SYS_UID;
  fproc[FS].fp_realgid=SYS_GID;
  fproc[FS].fp_realgid=SYS_GID;

  fproc[INIT].fp_umask=~0;
  fproc[INIT].fp_workdir=super_blocks[0].s_isup;
  fproc[INIT].fp_rootdir=super_blocks[0].s_isup;

  for (i=0; i<NR_FDS; i++)
    fproc[INIT].fp_filp[i]=NIL_FILP;
  fproc[INIT].fp_suspended=NOT_SUSPENDED;
  fproc[INIT].fp_revived=NOT_REVIVING;
  fproc[INIT].fp_realuid=SYS_UID;
  fproc[INIT].fp_effuid=SYS_UID;
  fproc[INIT].fp_realgid=SYS_GID;
  fproc[INIT].fp_realgid=SYS_GID;

  for (i=0; i<6; i++)
    dup_inode(super_blocks[0].s_isup);
/* root_dir (z ROOT_INODE, pak dup(ROOT_INODE) ), work_dir (dup predchoziho)
 ef. a real UID,GID: vyhrazene hodnoty SYS_UID, SYS_GID
 umask=~0 */
}

/*===========================================================================*
 *                              load_super                                   *
 *===========================================================================*/
PRIVATE void load_super() /* place to get superblock from */
{
 struct inode *pinoder;
 int i;
 for (i=1; i<NR_SUPERS; i++)
   super_blocks[i].s_dev=NO_DEV;
 /* inicializace super_block[] na prazdnou */
 super_blocks[0].s_dev=ROOT_DEV;

  #ifdef REPORT_FS_INIT
   kprintf(INITREPORT_ROW,initReportCol,7,"%c",'0'+(initReportCol++-INITREPORT_COL));
  #endif

 rw_super(&super_blocks[0], READING);
 /* nacti superblock ROOT_DEV, inicializuj jeho pametove polozky, */

  #ifdef REPORT_FS_INIT
   kprintf(INITREPORT_ROW,initReportCol,7,"%c",'0'+(initReportCol++-INITREPORT_COL));
  #endif

 load_bit_maps(super_blocks[0].s_dev);

  #ifdef REPORT_FS_INIT
   kprintf(INITREPORT_ROW,initReportCol,7,"%c",'0'+(initReportCol++-INITREPORT_COL));
  #endif

 pinoder=get_inode(super_blocks[0].s_dev,ROOT_INODE);

  #ifdef REPORT_FS_INIT
   kprintf(INITREPORT_ROW,initReportCol,7,"%c",'0'+(initReportCol++-INITREPORT_COL));
  #endif

 super_blocks[0].s_imount=NIL_INODE;
 super_blocks[0].s_isup=pinoder;
 super_blocks[0].s_rd_only=0;
 /* nacti inode ROOT_DIR, load_bitmaps() */
}

/*===========================================================================*
 *                              fetch_name                                   *
/*===========================================================================*/
PUBLIC void fetch_name(vir_bytes virtStrAddr)
{
/*  Prekopiruje string na adrese virtStrAddr procesu indikovaneho *fp
  do user_path */
  static message msg_sys;
  char* dest=user_path;

  do {
   msg_sys.m_type=SYS_COPY;
   msg_sys.SRC_SEG=D;
   msg_sys.SRC_PROC=who;
   msg_sys.SRC_VIR=virtStrAddr++;

   msg_sys.DST_SEG=D;
   msg_sys.DST_PROC=FS;
   msg_sys.DST_VIR=(vir_bytes) dest;

   msg_sys.BYTE_CNT=1;
   send_recv(SYSTASK, msg_sys);
  } while (*dest++!=0);
}

/*===========================================================================*
 *                              main                                         *
 *===========================================================================*/

#if (TEST_PHASE>MAX_PHASE_KERNEL_MM || TEST_PHASE==0)
PUBLIC void main()
#else
PUBLIC void fs_main()
#endif
{
/* This is the main program of the file system.  The main loop consists of
 * three major activities: getting new work, processing the work, and sending
 * the reply.  This loop never terminates as long as the file system runs.
 */

  #ifdef REPORT_FS_INIT
   kprintf(INITREPORT_ROW,0,7,"Initializing FS: ");
  #endif


  fs_init();

/* inicializace IPX driveru - pokud toto volani
   odebereme vyradime IPX driver ze hry		*/
  ipx_init();

  #ifdef REPORT_FS_INIT
   kprintf(INITREPORT_ROW,initReportCol,7,"%c",'0'+(initReportCol++-INITREPORT_COL));
  #endif

  while (TRUE) {

    receive(ANY,fs_msg)

    /* podle odesilatele message nastav who, fp a fs_call) */
    who=fs_msg.m_source;
    fs_call=fs_msg.m_type;
    fp=&fproc[who];
    dont_reply = 0;
    fs_repmsg.m_type = TASK_REPLY;

    superuser = (fp->fp_effuid==SU_UID || fp->fp_effgid==SU_UID)? TRUE:FALSE;

/* odbocka pro IPX */
	if ((fs_call == C_SOCKET) || (fs_call == C_NOTIFY ) || 
	    (fs_call == C_SETLNET) || (fs_call == C_SETLNODE) || (fs_call == C_SETGATEWAY) ||
	    ((fs_call == C_READ || fs_call == C_WRITE || fs_call == C_CLOSE) &&
		  fs_msg.fs_fd >= IPX_SOCK_PREF && ipx_ready) 							)
		switch (fs_call) {
			case C_SOCKET:		ipx_do_socket();
						break;
			case C_NOTIFY:		ipx_do_notify();
						break;
			case C_READ:		ipx_do_read();
						break;
			case C_WRITE:		ipx_do_write();
						break;
			case C_CLOSE:		ipx_do_close();
						break;
			case C_SETLNET:		ipx_set_lnet();
						break;
			case C_SETLNODE:	ipx_set_lnode();
						break;
			case C_SETGATEWAY:	ipx_set_gateway();
		} 
/* Vyvolani funkce odpovidajici zprave */
    	else (*call_vector[fs_call])();

    /* Copy the results back to the user and send reply. */
    if (dont_reply) continue;
      reply(who);
  }
}

#if (TEST_PHASE>MAX_PHASE_KERNEL_MM || TEST_PHASE==0)
void exit(int i) { }
#endif
